import { convertVideo } from "$lib/ffmpeg.server";
import { error, json, text } from "@sveltejs/kit";
import type { RequestHandler } from "./$types";
import { PutObjectCommand } from "@aws-sdk/client-s3";
import { env } from "$env/dynamic/private";
import { s3 } from "$lib/storage.server";
import { hexToUint8, verify } from "$lib/crypto";

// This route is currently vulnerable against replay attacks across multiple days
// -- possible fix: prepend date to signature verification payload
export const POST: RequestHandler = async ({ request, url }) => {
    const pubkeyHex = url.searchParams.get('pubkey')?.trim()?.toLowerCase();
    if (!pubkeyHex || !/^[0-9a-f]{64}$/.test(pubkeyHex)) error(400);
    const pubKey = hexToUint8(pubkeyHex);

    const signatureHex = url.searchParams.get('signature')?.trim()?.toLowerCase();
    if (!signatureHex || !/^[0-9a-f]+$/.test(signatureHex)) error(400);
    const signature = hexToUint8(signatureHex);

    const today = new Date().toISOString().slice(0, 10);

    const bodyBuffer = await request.arrayBuffer();
    const bodyUint8 = new Uint8Array(bodyBuffer);

    const valid = await verify(pubKey, signature, bodyUint8);
    if (!valid) error(401);

    const command = new PutObjectCommand({
        Bucket: env.S3_BUCKET,
        Key: `${pubkeyHex}/${today}`,
        Body: bodyUint8,
        ContentType: 'application/octet-stream',
    });

    await s3.send(command);

    return text('ok');
}
